<?php
/* --------------------------------------------------------------
   HubConfigurationAjaxHandler.inc.php 2017-02-28
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2017 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

use \HubPublic\Exceptions\CurlRequestException;
use \HubPublic\Http\CurlRequest;

/**
 * Class HubConfigurationAjaxHandler
 *
 * Handles the Gambio Hub configuration AJAX requests.
 */
class HubConfigurationAjaxHandler extends AjaxHandler
{
	/**
	 * Returns the Permission Status
	 *
	 * @param int $customerId Optional, customer ID that makes the request.
	 *
	 * @return bool Returns whether the module is accessible or not.
	 */
	public function get_permission_status($customerId = null)
	{
		return true;
	}
	
	
	/**
	 * Proceeds with the module execution.
	 *
	 * @return bool Determines if the process was executed successfully.
	 */
	public function proceed()
	{
		switch($_GET['action'])
		{
			case 'get_shop_data':
				$response = $this->get_shop_data();
				break;
			
			case 'get_translations':
				$response = $this->get_translations();
				break;
			
			case 'create_client_key':
				$response = $this->create_client_key();
				break;
			
			case 'create_session_key':
				$response = $this->create_session_key();
				break;
			
			case 'get_order_statuses':
				$response = $this->get_order_statuses();
				break;
			
			default:
				$response = [];
		}
		
		$this->v_output_buffer = json_encode($response);
		
		return true;
	}
	
	
	/**
	 * Get the shop data.
	 *
	 * If a Hub Client Key exists, it will be returned instead of the shop data so that the merchant
	 * data can be loaded from the customer portal.
	 *
	 * @return array
	 */
	public function get_shop_data()
	{
		try
		{
			$customerData = $this->_get_row_data('customers', 'customers_id', $_SESSION['customer_id']);
			$zoneData     = $this->_get_row_data('zones', 'zone_id', STORE_ZONE);
			$countryData  = $this->_get_row_data('countries', 'countries_id', $zoneData['zone_country_id']);

            // Authentication hash.
            $authHash = AuthHashCreator::create();
			
			$response = [
				'gender'      => (string)$customerData['customers_gender'],
				'firstname'   => TRADER_FIRSTNAME,
				'lastname'    => TRADER_NAME,
				'street'      => TRADER_STREET,
                'housenumber' => TRADER_STREET_NUMBER,
                'city'        => TRADER_LOCATION,
                'state'       => (string)$zoneData['zone_name'],
				'postcode'    => TRADER_ZIPCODE,
				'country'     => (string)$countryData['countries_name'],
				'company'     => COMPANY_NAME,
				'email'       => (string)$customerData['customers_email_address'],
				'telephone'   => TRADER_TEL,
				'shopname'    => STORE_NAME,
				'shopkey'     => GAMBIO_SHOP_KEY,
                'authhash'    => $authHash->asString(),
                'version'     => ltrim(gm_get_conf('INSTALLED_VERSION'), 'v'),
                'url'         => HTTP_SERVER . DIR_WS_CATALOG
			];
			
			$hubClientKeyConfiguration = MainFactory::create('HubClientKeyConfiguration');
			
			$response['clientkey'] = $hubClientKeyConfiguration->get();
		}
		catch(Exception $exception)
		{
			// The app will continue in demo mode.
		}
		
		return $response;
	}
	
	
	/**
	 * Starts the creation of the hub client key.
	 */
	public function create_client_key()
	{
		// Authentication hash.
		$authHash = AuthHashCreator::create();
		
		// Hub API URL.
		$url = MODULE_PAYMENT_GAMBIO_HUB_URL;
		
		// Hub shop key configuration instance.
		$shopKeyConfiguration = MainFactory::create('HubShopKeyConfiguration');
		
		// Hub client key configuration instance.
		$clientKeyConfiguration = MainFactory::create('HubClientKeyConfiguration');
		
		// cUrl request instance.
		$curlRequest = new CurlRequest();
		
		// Shop logger instance. 
		$logControl = LogControl::get_instance();
		
		// Hub client keys API client.
		$clientKeysApiClient = MainFactory::create('HubClientKeysApiClient', $url, $shopKeyConfiguration,
		                                           $clientKeyConfiguration, $curlRequest, $logControl);
		
		// Fetch shop information. 
		$shopUrl     = HTTP_SERVER . DIR_WS_CATALOG;
		$shopVersion = ltrim(gm_get_conf('INSTALLED_VERSION'), 'v');
		
		try
		{
			$clientKeysApiClient->createClientKey($authHash, $shopUrl, $shopVersion);
			$response = ['success' => true];
			
			$sql = '
				UPDATE
					configuration
				SET
					configuration_value = "True" 
				WHERE
					configuration_key = "MODULE_PAYMENT_GAMBIO_HUB_STATUS"';
			xtc_db_query($sql, 'db_link', false);
		}
		catch(RuntimeException $exception)
		{
			// Send error.
			http_response_code(500);
			$response = AjaxException::response($exception);
		}
		
		return $response;
	}
	
	
	/**
	 * Starts the creation of the session key.
	 */
	public function create_session_key()
	{
		// Start a new gambio hub session
		$serviceFactory         = MainFactory::create('HubServiceFactory');
		$sessionKeyService      = $serviceFactory->createHubSessionKeyService();
		$clientKeyConfiguration = MainFactory::create('HubClientKeyConfiguration');
		$curlRequest            = new CurlRequest();
		$logControl             = LogControl::get_instance();
		$sessionsApiClient      = MainFactory::create('HubSessionsApiClient', MODULE_PAYMENT_GAMBIO_HUB_URL,
		                                              $sessionKeyService, $clientKeyConfiguration, $curlRequest,
		                                              $logControl);
		$authHash               = AuthHashCreator::create();
		
		try
		{
			$response['gambio_hub_session_key'] = $sessionsApiClient->startSession($authHash);
		}
		catch(UnexpectedValueException $exception)
		{
			// Send error.
			http_response_code(500);
			$response = AjaxException::response($exception);
		}
		catch(CurlRequestException $exception)
		{
			// Send error.
			http_response_code(500);
			$response = AjaxException::response($exception);
		}
		
		return $response;
	}
	
	
	/**
	 * Get the Gambio Hub Translations
	 *
	 * @return array|string
	 */
	public function get_translations()
	{
		try
		{
			$languageTextManager = MainFactory::create('LanguageTextManager', 'gambio_hub_account',
			                                           $_SESSION['languages_id']);
			$response            = $languageTextManager->get_section_array();
		}
		catch(Exception $exception)
		{
			http_response_code(500);
			$response = AjaxException::response($exception);
		}
		
		return $response;
	}
	
	
	/**
	 * Returns all order status and the default one.
	 *
	 * @return array Returns the order status data.
	 */
	public function get_order_statuses()
	{
		// Response.
		$response = [];
		
		// Order status key-value array (ID is the key, and name is the value).
		$availableOrderStatuses = [];
		
		// Order status fetch query.
		$orderStatuses = $this->_get_table_data('orders_status', 'language_id = ' . $_SESSION['languages_id']);
		
		// Iterate over each query result row and append the ID and name to the order status array.
		foreach($orderStatuses as $orderStatus)
		{
			// Order status ID.
			$id = (int)$orderStatus['orders_status_id'];
			
			// Order status name.
			$name = (string)$orderStatus['orders_status_name'];
			
			// Append data to order status array.
			$availableOrderStatuses[$id] = $name;
		}
		
		// Append data to response.
		$response['default'] = (int)DEFAULT_ORDERS_STATUS_ID;
		$response['all']     = $availableOrderStatuses;
		
		return $response;
	}
	
	
	/**
	 * Updates Hub payment modules configuration.
	 *
	 * Payment module configuration values are necessary in various shop sections and therefore this method will
	 * update them.
	 *
	 * Provide the following POST values:
	 *
	 *   - moduleCode >> Contains the module code to be updated.
	 *   - moduleConfiguration >> Contains the configuration key-value data.
	 *
	 * This method can be extended further for more configuration values.
	 */
	public function update_hub_module_configuration()
	{
		try
		{
			$body = json_decode(file_get_contents('php://input'), true);
			
			$this->_update_hub_payment_module_alias($body['moduleCode'], $body['moduleConfiguration']['alias']);
			
			$response = ['success' => true];
		}
		catch(Exception $exception)
		{
			http_response_code(500);
			$response = AjaxException::response($exception);
		}
		
		return MainFactory::create('JsonHttpControllerResponse', $response);
	}
	
	
	/**
	 * Updates/Inserts alias value for Gambio Hub module.
	 *
	 * @param string $moduleCode Gambio Hub module code.
	 * @param string $alias      New module alias string.
	 */
	protected function _update_hub_payment_module_alias($moduleCode, $alias)
	{
		$configurationKey = 'MODULE_PAYMENT_GAMBIO_HUB_' . strtoupper($moduleCode) . '_ALIAS';
		
		gm_set_conf($configurationKey, $alias);
	}
	
	
	/**
	 * Get Table Data
	 *
	 * Returns the fetched data from the database if entries were found, else
	 * an empty array will be returned.
	 *
	 * @param string $tableName  Name of the table from the database.
	 * @param string $columnName Name of the identifier to fetch the data from such as 'customers_id'.
	 * @param int    $rowId      ID of the entry to fetch the data from.
	 *
	 * @return array Fetched data.
	 */
	protected function _get_row_data($tableName, $columnName, $rowId)
	{
		$sql = 'SELECT * FROM ' . $tableName . ' WHERE ' . $columnName . ' = ' . $rowId;
		
		$query = xtc_db_query($sql);
		
		return xtc_db_fetch_array($query);
	}
	
	
	/**
	 * Get Table Data
	 *
	 * Returns the fetched data from the database if entries were found, else
	 * an empty array will be returned.
	 *
	 * @param string $tableName  Name of the table from the database.
	 * @param string $conditions The row selection conditions.
	 *
	 * @return array Fetched data.
	 */
	protected function _get_table_data($tableName, $conditions = '')
	{
		$sql = 'SELECT * FROM ' . $tableName;
		
		if(!empty($conditions))
		{
			$sql .= ' WHERE ' . $conditions;
		}
		
		$query = xtc_db_query($sql);
		
		$results = [];
		
		while($row = xtc_db_fetch_array($query))
		{
			$results[] = $row;
		}
		
		return $results;
	}
}