<?php
/* --------------------------------------------------------------
   HubSessionsApiClient.inc.php 2017-02-25
   Gambio GmbH
   http://www.gambio.de
   Copyright © 2017 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

use \HubPublic\Http\CurlRequest;
use \HubPublic\ValueObjects\AuthHash;

/**
 * Class HubSessionApiClient
 *
 * @category   System
 * @package    Extensions
 * @subpackage GambioHub
 */
class HubSessionsApiClient implements HubSessionsApiClientInterface
{
	/**
	 * @var string
	 */
	protected $url;
	
	/**
	 * @var \HubSessionKeyService
	 */
	protected $hubSessionKeyService;
	
	/**
	 * @var \HubClientKeyConfiguration
	 */
	protected $clientKeyConfiguration;
	
	/**
	 * @var \HubPublic\Http\CurlRequest
	 */
	protected $curlRequest;
	
	/**
	 * @var \LogControl
	 */
	protected $logControl;
	
	
	/**
	 * HubSessionApiClient constructor.
	 *
	 * @param string                      $url                                       Gambio Hub API URL
	 * @param \HubSessionKeyService       $hubSessionKeyService                      Used for retrieving the session
	 *                                                                               key.
	 * @param \HubClientKeyConfiguration  $hubClientKeyConfiguration                 Stores or retrieves the hub client
	 *                                                                               key from shop configuration.
	 * @param \HubPublic\Http\CurlRequest $curlRequest                               Make cURL requests to the Hub API.
	 * @param \LogControl                 $logControl                                Log communication error
	 *                                                                               information.
	 *
	 * @throws InvalidArgumentException If the $url argument is not a valid URL.
	 */
	public function __construct($url,
	                            HubSessionKeyService $hubSessionKeyService,
	                            HubClientKeyConfiguration $hubClientKeyConfiguration,
	                            CurlRequest $curlRequest,
	                            LogControl $logControl)
	{
		if(!filter_var($url, FILTER_VALIDATE_URL))
		{
			throw new InvalidArgumentException('Invalid Gambio Hub API URL provided: ' . $url);
		}
		
		$this->url                    = $url;
		$this->clientKeyConfiguration = $hubClientKeyConfiguration;
		$this->hubSessionKeyService   = $hubSessionKeyService;
		$this->curlRequest            = $curlRequest;
		$this->logControl             = $logControl;
	}
	
	
	/**
	 * Starts a session in the Gambio Hub.
	 *
	 * Provide an authorization hash that can be later used in the hub callbacks to determine where each
	 * session key belongs to. This method will additionally save the AuthHash value to the PHP session with
	 * the key 'gambio_hub_auth_hash' for later reference.
	 *
	 * @param \HubPublic\ValueObjects\AuthHash $authHash The authorization hash to be used for the session start.
	 *
	 * @return string Returns the new session key.
	 *
	 * @throws UnexpectedValueException If the server responses with status code different to 201.
	 */
	public function startSession(AuthHash $authHash)
	{
		$clientKey = $this->clientKeyConfiguration->get();
		
		$requestUrl = $this->url . '/hub_client_keys/' . $clientKey;
		
		// Set parameter request URL.
		$this->curlRequest->setUrl($requestUrl);
		
		// Set parameter request data.
		$this->curlRequest->setOption(CURLOPT_HTTPHEADER, ['Expect:', 'X-Auth-Hash: ' . $authHash->asString()]);
		
		// Set parameter request method to 'POST'.
		$this->curlRequest->setOption(CURLOPT_POST, true);
		
		// Set empty body to avoid bug in old curl lib versions
		$this->curlRequest->setOption(CURLOPT_POSTFIELDS, '');
		
		// Set 10 seconds timeout
		$this->curlRequest->setOption(CURLOPT_TIMEOUT, 10);
		
		// Perform parameter request.
		$httpResponse = $this->curlRequest->execute();
		
		if($httpResponse->getStatusCode() !== 201)
		{
			$this->logControl->notice('Failed to start a new session. ' . $httpResponse->getBody(), '', 'hub',
			                          'notice', 'USER NOTICE', $httpResponse->getStatusCode());
			
			throw new UnexpectedValueException('Failed to start a new session. ' . $httpResponse->getBody() . ' ('
			                                   . $httpResponse->getStatusCode() . ')');
		}
		
		$sessionKey = $this->hubSessionKeyService->findByAuthHash($authHash); 
		
		$_SESSION['gambio_hub_session_key'] = $sessionKey ? $sessionKey->asString() : '';
		
		return $_SESSION['gambio_hub_session_key'];
	}
}